# Generated by roxygen2: do not edit by hand

S3method(as.ATSP,dist)
S3method(as.ATSP,matrix)
S3method(as.ETSP,data.frame)
S3method(as.ETSP,matrix)
S3method(as.TOUR,integer)
S3method(as.TOUR,numeric)
S3method(as.TSP,ETSP)
S3method(as.TSP,dist)
S3method(as.TSP,matrix)
S3method(as.dist,TSP)
S3method(as.matrix,ATSP)
S3method(as.matrix,ETSP)
S3method(cut_tour,TOUR)
S3method(image,ATSP)
S3method(image,ETSP)
S3method(image,TSP)
S3method(insert_dummy,ATSP)
S3method(insert_dummy,ETSP)
S3method(insert_dummy,TSP)
S3method(labels,ATSP)
S3method(labels,ETSP)
S3method(labels,TSP)
S3method(n_of_cities,ATSP)
S3method(n_of_cities,ETSP)
S3method(n_of_cities,TSP)
S3method(n_of_cities,default)
S3method(plot,ETSP)
S3method(print,ATSP)
S3method(print,ETSP)
S3method(print,TOUR)
S3method(print,TSP)
S3method(solve_TSP,ATSP)
S3method(solve_TSP,ETSP)
S3method(solve_TSP,TSP)
S3method(tour_length,ATSP)
S3method(tour_length,ETSP)
S3method(tour_length,TOUR)
S3method(tour_length,TSP)
S3method(tour_length,integer)
S3method(write_TSPLIB,ATSP)
S3method(write_TSPLIB,ETSP)
S3method(write_TSPLIB,TSP)
export(ATSP)
export(ETSP)
export(TOUR)
export(TSP)
export(as.ATSP)
export(as.ETSP)
export(as.TOUR)
export(as.TSP)
export(concorde_help)
export(concorde_path)
export(cut_tour)
export(filter_ATSP_as_TSP_dummies)
export(insert_dummy)
export(linkern_help)
export(n_of_cities)
export(read_TSPLIB)
export(reformulate_ATSP_as_TSP)
export(solve_TSP)
export(tour_length)
export(write_TSPLIB)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(grDevices,gray.colors)
importFrom(graphics,image.default)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(stats,as.dist)
importFrom(stats,dist)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,write.table)
useDynLib(TSP, .registration=TRUE)
